@@ -178,7 +178,7 @@ impl OpCode {
         self.0
     }
 }
-
+#[derive(Debug)]
 pub struct OpInfo {
     pub gas: u64,
     pub gas_block_end: bool,
@@ -187,7 +187,7 @@ pub struct OpInfo {
 impl OpInfo {
     pub const fn none() -> Self {
         Self {
-            gas: u64::MAX,
+            gas: 0,
             gas_block_end: true,
         }
     }
