@@ -1,6 +1,6 @@
 use super::gas;
 use crate::{
-    machine::Machine, CallContext, CallScheme, CreateScheme, Handler, Return, Spec, Transfer,
+    machine::Machine, CallContext, CallScheme, CreateScheme, Host, Return, Spec, Transfer,
 };
 use crate::{return_ok, return_revert};
 // 	CallScheme, Capture, CallContext, CreateScheme, ,
@@ -31,17 +31,17 @@ pub fn sha3(machine: &mut Machine) -> Return {
     Return::Continue
 }
 
-pub fn chainid<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut H) -> Return {
+pub fn chainid<H: Host, SPEC: Spec>(machine: &mut Machine, host: &mut H) -> Return {
     check!(SPEC::enabled(ISTANBUL)); // EIP-1344: ChainID opcode
-    gas!(machine, gas::BASE);
+                                     //gas!(machine, gas::BASE);
 
-    push!(machine, handler.env().cfg.chain_id);
+    push!(machine, host.env().cfg.chain_id);
 
     Return::Continue
 }
 
 pub fn address(machine: &mut Machine) -> Return {
-    gas!(machine, gas::BASE);
+    //gas!(machine, gas::BASE);
 
     let ret = H256::from(machine.contract.address);
     push_h256!(machine, ret);
@@ -49,9 +49,9 @@ pub fn address(machine: &mut Machine) -> Return {
     Return::Continue
 }
 
-pub fn balance<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut H) -> Return {
+pub fn balance<H: Host, SPEC: Spec>(machine: &mut Machine, host: &mut H) -> Return {
     pop_address!(machine, address);
-    let (balance, is_cold) = handler.balance(address);
+    let (balance, is_cold) = host.balance(address);
     gas!(
         machine,
         if SPEC::enabled(ISTANBUL) {
@@ -68,35 +68,34 @@ pub fn balance<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut H) -
     Return::Continue
 }
 
-pub fn selfbalance<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut H) -> Return {
+pub fn selfbalance<H: Host, SPEC: Spec>(machine: &mut Machine, host: &mut H) -> Return {
     check!(SPEC::enabled(ISTANBUL)); // EIP-1884: Repricing for trie-size-dependent opcodes
-    let (balance, _) = handler.balance(machine.contract.address);
-    gas!(machine, gas::LOW);
+                                     //gas!(machine, gas::LOW);
+    let (balance, _) = host.balance(machine.contract.address);
     push!(machine, balance);
 
     Return::Continue
 }
 
-pub fn basefee<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut H) -> Return {
+pub fn basefee<H: Host, SPEC: Spec>(machine: &mut Machine, host: &mut H) -> Return {
     check!(SPEC::enabled(LONDON)); // EIP-3198: BASEFEE opcode
-    let basefee = handler.env().block.basefee;
-    gas!(machine, gas::BASE);
-    push!(machine, basefee);
+                                   //gas!(machine, gas::BASE);
+    push!(machine, host.env().block.basefee);
 
     Return::Continue
 }
 
-pub fn origin<H: Handler>(machine: &mut Machine, handler: &mut H) -> Return {
-    gas!(machine, gas::BASE);
+pub fn origin<H: Host>(machine: &mut Machine, host: &mut H) -> Return {
+    //gas!(machine, gas::BASE);
 
-    let ret = H256::from(handler.env().tx.caller);
+    let ret = H256::from(host.env().tx.caller);
     push_h256!(machine, ret);
 
     Return::Continue
 }
 
 pub fn caller(machine: &mut Machine) -> Return {
-    gas!(machine, gas::BASE);
+    //gas!(machine, gas::BASE);
 
     let ret = H256::from(machine.contract.caller);
     push_h256!(machine, ret);
@@ -105,7 +104,7 @@ pub fn caller(machine: &mut Machine) -> Return {
 }
 
 pub fn callvalue(machine: &mut Machine) -> Return {
-    gas!(machine, gas::BASE);
+    //gas!(machine, gas::BASE);
 
     let mut ret = H256::default();
     machine.contract.value.to_big_endian(&mut ret[..]);
@@ -114,23 +113,16 @@ pub fn callvalue(machine: &mut Machine) -> Return {
     Return::Continue
 }
 
-pub fn gasprice<H: Handler>(machine: &mut Machine, handler: &mut H) -> Return {
-    gas!(machine, gas::BASE);
-
-    let mut ret = H256::default();
-    handler
-        .env()
-        .effective_gas_price()
-        .to_big_endian(&mut ret[..]);
-    push_h256!(machine, ret);
-
+pub fn gasprice<H: Host>(machine: &mut Machine, host: &mut H) -> Return {
+    //gas!(machine, gas::BASE);
+    push!(machine, host.env().effective_gas_price());
     Return::Continue
 }
 
-pub fn extcodesize<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut H) -> Return {
+pub fn extcodesize<H: Host, SPEC: Spec>(machine: &mut Machine, host: &mut H) -> Return {
     pop_address!(machine, address);
 
-    let (code, is_cold) = handler.code(address);
+    let (code, is_cold) = host.code(address);
     gas!(machine, gas::account_access_gas::<SPEC>(is_cold));
 
     push!(machine, U256::from(code.len()));
@@ -138,10 +130,10 @@ pub fn extcodesize<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut
     Return::Continue
 }
 
-pub fn extcodehash<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut H) -> Return {
+pub fn extcodehash<H: Host, SPEC: Spec>(machine: &mut Machine, host: &mut H) -> Return {
     check!(SPEC::enabled(CONSTANTINOPLE)); // EIP-1052: EXTCODEHASH opcode
     pop_address!(machine, address);
-    let (code_hash, is_cold) = handler.code_hash(address);
+    let (code_hash, is_cold) = host.code_hash(address);
     gas!(
         machine,
         if SPEC::enabled(ISTANBUL) {
@@ -156,11 +148,11 @@ pub fn extcodehash<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut
     Return::Continue
 }
 
-pub fn extcodecopy<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut H) -> Return {
+pub fn extcodecopy<H: Host, SPEC: Spec>(machine: &mut Machine, host: &mut H) -> Return {
     pop_address!(machine, address);
     pop!(machine, memory_offset, code_offset, len_u256);
 
-    let (code, is_cold) = handler.code(address);
+    let (code, is_cold) = host.code(address);
     gas_or_fail!(machine, gas::extcodecopy_cost::<SPEC>(len_u256, is_cold));
     let len = as_usize_or_fail!(len_u256, Return::OutOfGas);
     if len == 0 {
@@ -178,7 +170,7 @@ pub fn extcodecopy<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut
 
 pub fn returndatasize<SPEC: Spec>(machine: &mut Machine) -> Return {
     check!(SPEC::enabled(BYZANTINE)); // EIP-211: New opcodes: RETURNDATASIZE and RETURNDATACOPY
-    gas!(machine, gas::BASE);
+                                      //gas!(machine, gas::BASE);
 
     let size = U256::from(machine.return_data_buffer.len());
     push!(machine, size);
@@ -206,80 +198,62 @@ pub fn returndatacopy<SPEC: Spec>(machine: &mut Machine) -> Return {
     Return::Continue
 }
 
-pub fn blockhash<H: Handler>(machine: &mut Machine, handler: &mut H) -> Return {
-    gas!(machine, gas::BLOCKHASH);
+pub fn blockhash<H: Host>(machine: &mut Machine, host: &mut H) -> Return {
+    //gas!(machine, gas::BLOCKHASH);
 
     pop!(machine, number);
-    push_h256!(machine, handler.block_hash(number));
+    push_h256!(machine, host.block_hash(number));
 
     Return::Continue
 }
 
-pub fn coinbase<H: Handler>(machine: &mut Machine, handler: &mut H) -> Return {
-    gas!(machine, gas::BASE);
+pub fn coinbase<H: Host>(machine: &mut Machine, host: &mut H) -> Return {
+    //gas!(machine, gas::BASE);
 
-    push_h256!(machine, handler.env().block.coinbase.into());
+    push_h256!(machine, host.env().block.coinbase.into());
     Return::Continue
 }
 
-pub fn timestamp<H: Handler>(machine: &mut Machine, handler: &mut H) -> Return {
-    gas!(machine, gas::BASE);
-    push!(machine, handler.env().block.timestamp);
+pub fn timestamp<H: Host>(machine: &mut Machine, host: &mut H) -> Return {
+    //gas!(machine, gas::BASE);
+    push!(machine, host.env().block.timestamp);
     Return::Continue
 }
 
-pub fn number<H: Handler>(machine: &mut Machine, handler: &mut H) -> Return {
-    gas!(machine, gas::BASE);
+pub fn number<H: Host>(machine: &mut Machine, host: &mut H) -> Return {
+    //gas!(machine, gas::BASE);
 
-    push!(machine, handler.env().block.number);
+    push!(machine, host.env().block.number);
     Return::Continue
 }
 
-pub fn difficulty<H: Handler>(machine: &mut Machine, handler: &mut H) -> Return {
-    gas!(machine, gas::BASE);
+pub fn difficulty<H: Host>(machine: &mut Machine, host: &mut H) -> Return {
+    //gas!(machine, gas::BASE);
 
-    push!(machine, handler.env().block.difficulty);
+    push!(machine, host.env().block.difficulty);
     Return::Continue
 }
 
-pub fn gaslimit<H: Handler>(machine: &mut Machine, handler: &mut H) -> Return {
-    gas!(machine, gas::BASE);
+pub fn gaslimit<H: Host>(machine: &mut Machine, host: &mut H) -> Return {
+    //gas!(machine, gas::BASE);
 
-    push!(machine, handler.env().block.gas_limit);
+    push!(machine, host.env().block.gas_limit);
     Return::Continue
 }
 
-pub fn sload<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut H) -> Return {
+pub fn sload<H: Host, SPEC: Spec>(machine: &mut Machine, host: &mut H) -> Return {
     pop!(machine, index);
-    let (value, is_cold) = handler.sload(machine.contract.address, index);
-    inspect!(
-        handler,
-        sload,
-        &machine.contract.address,
-        &index,
-        &value,
-        is_cold
-    );
+    let (value, is_cold) = host.sload(machine.contract.address, index);
     gas!(machine, gas::sload_cost::<SPEC>(is_cold));
     push!(machine, value);
     Return::Continue
 }
 
-pub fn sstore<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut H) -> Return {
+pub fn sstore<H: Host, SPEC: Spec>(machine: &mut Machine, host: &mut H) -> Return {
     check!(!SPEC::IS_STATIC_CALL);
 
     pop!(machine, index, value);
-    let (original, old, new, is_cold) = handler.sstore(machine.contract.address, index, value);
-    inspect!(
-        handler,
-        sstore,
-        machine.contract.address,
-        index,
-        new,
-        old,
-        original,
-        is_cold
-    );
+    let (original, old, new, is_cold) = host.sstore(machine.contract.address, index, value);
     gas_or_fail!(machine, {
         let remaining_gas = machine.gas.remaining();
         gas::sstore_cost::<SPEC>(original, old, new, remaining_gas, is_cold)
@@ -289,13 +263,13 @@ pub fn sstore<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut H) ->
 }
 
 pub fn gas(machine: &mut Machine) -> Return {
-    gas!(machine, gas::BASE);
+    //gas!(machine, gas::BASE);
 
     push!(machine, U256::from(machine.gas.remaining()));
-    Return::Continue
+    machine.add_next_gas_block(machine.program_counter() - 1)
 }
 
-pub fn log<H: Handler, SPEC: Spec>(machine: &mut Machine, n: u8, handler: &mut H) -> Return {
+pub fn log<H: Host, SPEC: Spec>(machine: &mut Machine, n: u8, host: &mut H) -> Return {
     check!(!SPEC::IS_STATIC_CALL);
 
     pop!(machine, offset, len);
@@ -321,16 +295,15 @@ pub fn log<H: Handler, SPEC: Spec>(machine: &mut Machine, n: u8, handler: &mut H
         topics.push(t);
     }
 
-    handler.log(machine.contract.address, topics, data);
+    host.log(machine.contract.address, topics, data);
     Return::Continue
 }
 
-pub fn selfdestruct<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut H) -> Return {
+pub fn selfdestruct<H: Host, SPEC: Spec>(machine: &mut Machine, host: &mut H) -> Return {
     check!(!SPEC::IS_STATIC_CALL);
     pop_address!(machine, target);
 
-    let res = handler.selfdestruct(machine.contract.address, target);
-    inspect!(handler, selfdestruct);
+    let res = host.selfdestruct(machine.contract.address, target);
 
     // EIP-3529: Reduction in refunds
     if !SPEC::enabled(LONDON) && !res.previously_destroyed {
@@ -341,7 +314,6 @@ pub fn selfdestruct<H: Handler, SPEC: Spec>(machine: &mut Machine, handler: &mut
     Return::SelfDestruct
 }
 
-#[inline(always)]
 fn gas_call_l64_after<SPEC: Spec>(machine: &mut Machine) -> Result<u64, Return> {
     if SPEC::enabled(TANGERINE) {
         //EIP-150: Gas cost changes for IO-heavy operations
@@ -352,10 +324,10 @@ fn gas_call_l64_after<SPEC: Spec>(machine: &mut Machine) -> Result<u64, Return>
     }
 }
 
-pub fn create<H: Handler, SPEC: Spec>(
+pub fn create<H: Host, SPEC: Spec>(
     machine: &mut Machine,
     is_create2: bool,
-    handler: &mut H,
+    host: &mut H,
 ) -> Return {
     check!(!SPEC::IS_STATIC_CALL);
     if is_create2 {
@@ -388,38 +360,27 @@ pub fn create<H: Handler, SPEC: Spec>(
     let gas_limit = try_or_fail!(gas_call_l64_after::<SPEC>(machine));
     gas!(machine, gas_limit);
 
-    inspect!(
-        handler,
-        create,
-        machine.contract.address,
-        &scheme,
-        value,
-        &code,
-        gas_limit
-    );
-
     let (reason, address, gas, return_data) =
-        handler.create::<SPEC>(machine.contract.address, scheme, value, code, gas_limit);
+        host.create::<SPEC>(machine.contract.address, scheme, value, code, gas_limit);
     machine.return_data_buffer = return_data;
     let created_address: H256 = if matches!(reason, return_ok!()) {
         address.map(|a| a.into()).unwrap_or_default()
     } else {
         H256::default()
     };
-    inspect!(handler, create_return, created_address);
     push_h256!(machine, created_address);
     // reimburse gas that is not spend
     machine.gas.reimburse_unspend(&reason, gas);
     match reason {
         Return::FatalNotSupported => Return::FatalNotSupported,
-        _ => Return::Continue,
+        _ => machine.add_next_gas_block(machine.program_counter() - 1),
     }
 }
 
-pub fn call<H: Handler, SPEC: Spec>(
+pub fn call<H: Host, SPEC: Spec>(
     machine: &mut Machine,
     scheme: CallScheme,
-    handler: &mut H,
+    host: &mut H,
 ) -> Return {
     match scheme {
         CallScheme::DelegateCall => check!(SPEC::enabled(HOMESTEAD)), // EIP-7: DELEGATECALL
@@ -511,7 +472,7 @@ pub fn call<H: Handler, SPEC: Spec>(
     };
 
     // load account and calculate gas cost.
-    let (is_cold, exist) = handler.load_account(to);
+    let (is_cold, exist) = host.load_account(to);
     let is_new = !exist;
     //let is_cold = false;
     gas!(
@@ -536,13 +497,12 @@ pub fn call<H: Handler, SPEC: Spec>(
         gas_limit = gas_limit.saturating_add(gas::CALL_STIPEND);
     }
     let is_static = matches!(scheme, CallScheme::StaticCall);
-    inspect!(handler, call, to, &context, &transfer, &input, gas_limit, is_static);
 
     // CALL CONTRACT, with static or ordinary spec.
     let (reason, gas, return_data) = if is_static {
-        handler.call::<SPEC::STATIC>(to, transfer, input, gas_limit, context)
+        host.call::<SPEC::STATIC>(to, transfer, input, gas_limit, context)
     } else {
-        handler.call::<SPEC>(to, transfer, input, gas_limit, context)
+        host.call::<SPEC>(to, transfer, input, gas_limit, context)
     };
     machine.return_data_buffer = return_data;
 
@@ -566,5 +526,5 @@ pub fn call<H: Handler, SPEC: Spec>(
             push!(machine, U256::zero());
         }
     }
-    Return::Continue
+    machine.add_next_gas_block(machine.program_counter() - 1)
 }
